//----------------------------------------------------------------
//  Time-stamp: <99/08/04 13:05 sutner> 
//				<00/09/17 12:33 mjs>
//
//	File:		graph-adj.cc
//	Purpose:	Implement a directed, weighted graph via an Adjacency Matrix
//----------------------------------------------------------------

#include <iomanip>
#include "graph-adj.h"
using namespace std;


const double Graph::NO_EDGE = 1.0e100;	//define NO_EDGE's value for class

Graph::Graph()
	: myNumEdges(0)
{}

Graph::Graph(int n)
	: myNumEdges(0), myA(n,n,NO_EDGE)
{}

void Graph::SetSize(int n) 
{
	myA.SetDimensions(n,n);
	for (int o = 0; o < myA.size(); ++o)
		for (int d = 0; d < myA.size(); ++d)
			myA[o][d] = NO_EDGE;
}

void Graph::AddEdge(int origin, int destination, double value) 
{
	if (myA[origin][destination] == NO_EDGE)
		++myNumEdges;
	myA[origin][destination] = value;
}

void Graph::RemoveEdge(int origin, int destination) 
{
	if (myA[origin][destination] != NO_EDGE)
		--myNumEdges;
	myA[origin][destination] = NO_EDGE;
}

bool Graph::HasEdge(int origin, int destination)  const
{
   return myA[origin][destination] != NO_EDGE;
}

double Graph::EdgeValue(int origin, int destination)  const
{
   return myA[origin][destination];
}

int Graph::NumNodes()  const
{
	return myA.size();
}

int Graph::NumEdges()  const
{
	return myNumEdges;
}

ostream& operator<<(ostream &os, const Graph &g)
{
	os << "    ";
	for (int d = 0; d < g.NumNodes(); ++d) 
		os << setw(4) << d;
	os << endl;
    
    for (int o = 0; o < g.NumNodes(); ++o)
    {
		os << setw(4) << o;
		for (int d = 0; d < g.NumNodes(); ++d)
			if (g.myA[o][d] == Graph::NO_EDGE)
				os << "    ";
			else
				os << setw(4) << g.myA[o][d];
     	os << endl;
    }
    
    return os;
}
