//----------------------------------------------------------------
// Time-stamp:	99/06/25 15:07:09 sutner
//				00/08/30 11:29:34 mjs
// 
// File:     matrix.h
// Purpose:  Two-dim matrix built by inheritance from vector<vector<Type>>
//----------------------------------------------------------------

#ifndef _MATRIX_H_
#define _MATRIX_H_

#include <iostream>
#include <vector>
using namespace std;


template<class Type>
class Matrix: public vector<vector<Type> >	// mind that space between >'s
{
	public:
    	Matrix() {}			    // unallocated vectors
    	Matrix(int n, int m);	// dimensions given
		Matrix(int n, int m, const Type &fillValue);	// and default fill value

    	int NumRows()  const {return size();}
    	int NumColumns()  const {return (*this)[0].size();}
  
    	void SetDimensions(int r, int c);

    	friend ostream& operator<<<Type>(ostream &out, const Matrix<Type> &mat);   
};


//----------------------------------------------------------------
// IMPLEMENTATION
// not in separate .cpp file because class is templated
//----------------------------------------------------------------

template<class Type>
Matrix<Type>::Matrix(int n, int m): vector<vector<Type> >(n)
{ 
    for (int i = 0; i < n; ++i)
		(*this)[i].resize(m);
}

template<class Type>
Matrix<Type>::Matrix(int n, int m, const Type &fillValue): vector<vector<Type> >(n)
{ 
    for (int i = 0; i < n; ++i)
    {
		vector<Type>& ri = (*this)[i];
		ri.resize(m);
		fill(ri.begin(), ri.end(), fillValue);
    }
}

template<class Type>
void Matrix<Type>::SetDimensions(int r, int c)
{
    resize(r);
    for (int i = 0; i < r; ++i)
		(*this)[i].resize(c);
}

template<class Type>
ostream& operator<<(ostream &out, const Matrix<Type> &mat)
{
    for (int i = 0; i < mat.NumRows(); ++i)
		for (int j = 0; j < mat.NumColumns(); ++j)
			out << setw(3) << mat[i][j];

    return out;
}

#endif

